@@ -39,6 +39,7 @@ struct io_log {
 
 struct io_piece {
 	struct list_head list;
+	struct fio_file *file;
 	unsigned long long offset;
 	unsigned int len;
 	int ddir;
@@ -73,6 +74,8 @@ struct io_u {
 	unsigned char seen;
 	unsigned char ddir;
 
+	struct fio_file *file;
+
 	struct list_head list;
 };
 
@@ -124,13 +127,32 @@ enum fio_filetype {
 	FIO_TYPE_CHAR,
 };
 
-enum fio_iotype {
+enum fio_ioengine_flags {
 	FIO_SYNCIO	= 1 << 0,
-	FIO_MMAPIO	= 1 << 1 | FIO_SYNCIO,
-	FIO_LIBAIO	= 1 << 2,
-	FIO_POSIXAIO	= 1 << 3,
-	FIO_SGIO	= 1 << 4,
-	FIO_SPLICEIO	= 1 << 5 | FIO_SYNCIO,
+	FIO_CPUIO	= 1 << 1,
+	FIO_MMAPIO	= 1 << 2,
+	FIO_RAWIO	= 1 << 3,
+};
+
+struct fio_file {
+	/*
+	 * A file may not be a file descriptor, let the io engine decide
+	 */
+	union {
+		unsigned long file_data;
+		int fd;
+	};
+	char *file_name;
+	void *mmap;
+	unsigned long long file_size;
+	unsigned long long real_file_size;
+	unsigned long long file_offset;
+	unsigned long long last_pos;
+
+	unsigned long *file_map;
+	unsigned int num_maps;
+
+	int fileno;
 };
 
 /*
@@ -138,16 +160,16 @@ enum fio_iotype {
  */
 struct thread_data {
 	char name[32];
-	char *file_name;
 	char *directory;
 	char verror[80];
 	pthread_t thread;
 	int thread_number;
 	int groupid;
 	enum fio_filetype filetype;
+	struct fio_file *files;
+	unsigned int nr_files;
+	unsigned int next_file;
 	int error;
-	int fd;
-	void *mmap;
 	pid_t pid;
 	char *orig_buffer;
 	size_t orig_buffer_size;
@@ -159,7 +181,6 @@ struct thread_data {
 
 	unsigned char sequential;
 	unsigned char odirect;
-	unsigned char create_file;
 	unsigned char invalidate_cache;
 	unsigned char create_serialize;
 	unsigned char create_fsync;
@@ -167,9 +188,12 @@ struct thread_data {
 	unsigned char sync_io;
 	unsigned char verify;
 	unsigned char use_thread;
+	unsigned char unlink;
 	unsigned char do_disk_util;
 	unsigned char override_sync;
 	unsigned char rand_repeatable;
+	unsigned char write_lat_log;
+	unsigned char write_bw_log;
 
 	unsigned int bs;
 	unsigned int min_bs;
@@ -178,13 +202,9 @@ struct thread_data {
 	unsigned int fsync_blocks;
 	unsigned int start_delay;
 	unsigned long timeout;
-	enum fio_iotype io_engine;
 	unsigned int overwrite;
 	unsigned int bw_avg_time;
 	unsigned int loops;
-	unsigned long long file_size;
-	unsigned long long real_file_size;
-	unsigned long long file_offset;
 	unsigned long long zone_size;
 	unsigned long long zone_skip;
 	enum fio_memtype mem_type;
@@ -215,15 +235,7 @@ struct thread_data {
 	 * IO engine hooks, contains everything needed to submit an io_u
 	 * to any of the available IO engines.
 	 */
-	void *io_data;
-	char io_engine_name[16];
-	int (*io_prep)(struct thread_data *, struct io_u *);
-	int (*io_queue)(struct thread_data *, struct io_u *);
-	int (*io_getevents)(struct thread_data *, int, int, struct timespec *);
-	struct io_u *(*io_event)(struct thread_data *, int);
-	int (*io_cancel)(struct thread_data *, struct io_u *);
-	void (*io_cleanup)(struct thread_data *);
-	int (*io_sync)(struct thread_data *);
+	struct ioengine_ops *io_ops;
 
 	/*
 	 * Current IO depth and list of free and busy io_u's.
@@ -245,21 +257,26 @@ struct thread_data {
 
 	unsigned long runtime[2];		/* msec */
 	unsigned long long io_size;
+	unsigned long long total_file_size;
+	unsigned long long start_offset;
 	unsigned long long total_io_size;
 
 	unsigned long long io_blocks[2];
 	unsigned long long io_bytes[2];
 	unsigned long long zone_bytes;
 	unsigned long long this_io_bytes[2];
-	unsigned long long last_pos;
 	volatile int mutex;
 
 	/*
 	 * State for random io, a bitmap of blocks done vs not done
 	 */
 	os_random_state_t random_state;
-	unsigned long *file_map;
-	unsigned int num_maps;
+
+	/*
+	 * CPU "io" cycle burner
+	 */
+	unsigned int cpuload;
+	unsigned int cpucycle;
 
 	/*
 	 * bandwidth and latency stats
@@ -307,19 +324,18 @@ struct thread_data {
 	struct list_head io_log_list;
 };
 
-#define td_verror(td, err)						\
+#define __td_verror(td, err, msg)					\
 	do {								\
 		int e = (err);						\
 		(td)->error = e;					\
-		snprintf(td->verror, sizeof(td->verror) - 1, "file:%s:%d, error=%s", __FILE__, __LINE__, strerror(e));	\
+		snprintf(td->verror, sizeof(td->verror) - 1, "file:%s:%d, error=%s", __FILE__, __LINE__, (msg));	\
 	} while (0)
 
-extern struct io_u *__get_io_u(struct thread_data *);
-extern void put_io_u(struct thread_data *, struct io_u *);
+
+#define td_verror(td, err)	__td_verror((td), (err), strerror((err)))
+#define td_vmsg(td, err, msg)	__td_verror((td), (err), (msg))
 
 extern int rate_quit;
-extern int write_lat_log;
-extern int write_bw_log;
 extern int exitall_on_terminate;
 extern int thread_number;
 extern int shm_id;
@@ -327,6 +343,8 @@ extern int groupid;
 extern int terse_output;
 extern FILE *f_out;
 extern FILE *f_err;
+extern char *fio_inst_prefix;
+extern int temp_stall_ts;
 
 extern struct thread_data *threads;
 
@@ -335,9 +353,9 @@ extern struct thread_data *threads;
 #define td_rw(td)		((td)->iomix != 0)
 
 #define BLOCKS_PER_MAP		(8 * sizeof(long))
-#define TO_MAP_BLOCK(td, b)	((b) - ((td)->file_offset / (td)->min_bs))
-#define RAND_MAP_IDX(td, b)	(TO_MAP_BLOCK(td, b) / BLOCKS_PER_MAP)
-#define RAND_MAP_BIT(td, b)	(TO_MAP_BLOCK(td, b) & (BLOCKS_PER_MAP - 1))
+#define TO_MAP_BLOCK(td, f, b)	((b) - ((f)->file_offset / (td)->min_bs))
+#define RAND_MAP_IDX(td, f, b)	(TO_MAP_BLOCK(td, f, b) / BLOCKS_PER_MAP)
+#define RAND_MAP_BIT(td, f, b)	(TO_MAP_BLOCK(td, f, b) & (BLOCKS_PER_MAP - 1))
 
 #define MAX_JOBS	(1024)
 
@@ -405,10 +423,13 @@ extern int setup_rate(struct thread_data *);
 /*
  * Time functions
  */
+extern void time_init(void);
 extern unsigned long utime_since(struct timeval *, struct timeval *);
 extern unsigned long mtime_since(struct timeval *, struct timeval *);
 extern unsigned long mtime_since_now(struct timeval *);
 extern unsigned long time_since_now(struct timeval *);
+extern unsigned long mtime_since_genesis(void);
+extern void __usec_sleep(unsigned int);
 extern void usec_sleep(struct thread_data *, unsigned long);
 extern void rate_throttle(struct thread_data *, unsigned long, unsigned int);
 
@@ -418,6 +439,70 @@ extern void rate_throttle(struct thread_data *, unsigned long, unsigned int);
 extern int parse_options(int, char **);
 extern int init_random_state(struct thread_data *);
 
+/*
+ * File setup/shutdown
+ */
+extern void close_files(struct thread_data *);
+extern int setup_files(struct thread_data *);
+extern int file_invalidate_cache(struct thread_data *, struct fio_file *);
+
+/*
+ * ETA/status stuff
+ */
+extern void print_thread_status(void);
+extern void print_status_init(int);
+
+/*
+ * Thread life cycle. Once a thread has a runstate beyond TD_INITIALIZED, it
+ * will never back again. It may cycle between running/verififying/fsyncing.
+ * Once the thread reaches TD_EXITED, it is just waiting for the core to
+ * reap it.
+ */
+enum {
+	TD_NOT_CREATED = 0,
+	TD_CREATED,
+	TD_INITIALIZED,
+	TD_RUNNING,
+	TD_VERIFYING,
+	TD_FSYNCING,
+	TD_EXITED,
+	TD_REAPED,
+};
+
+/*
+ * Verify helpers
+ */
+extern void populate_verify_io_u(struct thread_data *, struct io_u *);
+extern int get_next_verify(struct thread_data *td, struct io_u *);
+extern int do_io_u_verify(struct thread_data *, struct io_u **);
+
+/*
+ * Memory helpers
+ */
+extern int fio_pin_memory(void);
+extern void fio_unpin_memory(void);
+extern int allocate_io_mem(struct thread_data *);
+extern void free_io_mem(struct thread_data *);
+
+/*
+ * io unit handling
+ */
+#define queue_full(td)	list_empty(&(td)->io_u_freelist)
+extern struct io_u *__get_io_u(struct thread_data *);
+extern struct io_u *get_io_u(struct thread_data *, struct fio_file *);
+extern void put_io_u(struct thread_data *, struct io_u *);
+extern void ios_completed(struct thread_data *, struct io_completion_data *);
+extern void io_completed(struct thread_data *, struct io_u *, struct io_completion_data *);
+
+/*
+ * io engine entry points
+ */
+extern int td_io_init(struct thread_data *);
+extern int td_io_prep(struct thread_data *, struct io_u *);
+extern int td_io_queue(struct thread_data *, struct io_u *);
+extern int td_io_sync(struct thread_data *, struct fio_file *);
+extern int td_io_getevents(struct thread_data *, int, int, struct timespec *);
+
 /*
  * This is a pretty crappy semaphore implementation, but with the use that fio
  * has (just signalling start/go conditions), it doesn't have to be better.
@@ -451,4 +536,36 @@ static inline void fio_sem_up(volatile int *sem)
 		fprintf(stderr, ##args);	\
 	} while (0)
 
+struct ioengine_ops {
+	char name[16];
+	int version;
+	int flags;
+	int (*setup)(struct thread_data *);
+	int (*init)(struct thread_data *);
+	int (*prep)(struct thread_data *, struct io_u *);
+	int (*queue)(struct thread_data *, struct io_u *);
+	int (*getevents)(struct thread_data *, int, int, struct timespec *);
+	struct io_u *(*event)(struct thread_data *, int);
+	int (*cancel)(struct thread_data *, struct io_u *);
+	void (*cleanup)(struct thread_data *);
+	int (*sync)(struct thread_data *, struct fio_file *);
+	void *data;
+	void *dlhandle;
+};
+
+#define FIO_IOOPS_VERSION	2
+
+extern struct ioengine_ops *load_ioengine(struct thread_data *, char *);
+extern void close_ioengine(struct thread_data *);
+
+/*
+ * Mark unused variables passed to ops functions as unused, to silence gcc
+ */
+#define fio_unused	__attribute((__unused__))
+
+#define for_each_td(td, i)	\
+	for ((i) = 0, (td) = &threads[0]; (i) < (int) thread_number; (i)++, (td)++)
+#define for_each_file(td, f, i)	\
+	for ((i) = 0, (f) = &(td)->files[0]; (i) < (int) (td)->nr_files; (i)++, (f)++)
+
 #endif
