@@ -28,7 +28,6 @@
 #define DEF_SEQUENTIAL		(1)
 #define DEF_RAND_REPEAT		(1)
 #define DEF_OVERWRITE		(1)
-#define DEF_CREATE		(1)
 #define DEF_INVALIDATE		(1)
 #define DEF_SYNCIO		(0)
 #define DEF_RANDSEED		(0xb1899bedUL)
@@ -46,6 +45,10 @@
 #define DEF_RWMIX_CYCLE		(500)
 #define DEF_RWMIX_READ		(50)
 #define DEF_NICE		(0)
+#define DEF_NR_FILES		(1)
+#define DEF_UNLINK		(0)
+#define DEF_WRITE_BW_LOG	(0)
+#define DEF_WRITE_LAT_LOG	(0)
 
 static int def_timeout = DEF_TIMEOUT;
 
@@ -58,14 +61,15 @@ struct thread_data def_thread;
 struct thread_data *threads = NULL;
 
 int rate_quit = 0;
-int write_lat_log = 0;
-int write_bw_log = 0;
 int exitall_on_terminate = 0;
 int terse_output = 0;
 unsigned long long mlock_size = 0;
 FILE *f_out = NULL;
 FILE *f_err = NULL;
 
+static int write_lat_log = DEF_WRITE_LAT_LOG;
+static int write_bw_log = DEF_WRITE_BW_LOG;
+
 /*
  * Return a free job structure.
  */
@@ -82,7 +86,6 @@ static struct thread_data *get_new_job(int global, struct thread_data *parent)
 	*td = *parent;
 	td->name[0] = '\0';
 
-	td->fd = -1;
 	td->thread_number = thread_number;
 	return td;
 }
@@ -103,7 +106,8 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	char *ddir_str[] = { "read", "write", "randread", "randwrite",
 			     "rw", NULL, "randrw" };
 	struct stat sb;
-	int numjobs, ddir;
+	int numjobs, ddir, i;
+	struct fio_file *f;
 
 #ifndef FIO_HAVE_LIBAIO
 	if (td->io_engine == FIO_LIBAIO) {
@@ -124,17 +128,28 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	if (td == &def_thread)
 		return 0;
 
-	if (td->io_engine & FIO_SYNCIO)
+	/*
+	 * Set default io engine, if none set
+	 */
+	if (!td->io_ops) {
+		td->io_ops = load_ioengine(td, DEF_IO_ENGINE_NAME);
+		if (!td->io_ops) {
+			log_err("default engine %s not there?\n", DEF_IO_ENGINE_NAME);
+			return 1;
+		}
+	}
+
+	if (td->io_ops->flags & FIO_SYNCIO)
 		td->iodepth = 1;
 	else {
 		if (!td->iodepth)
-			td->iodepth = 1;
+			td->iodepth = td->nr_files;
 	}
 
 	/*
-	 * only really works for sequential io for now
+	 * only really works for sequential io for now, and with 1 file
 	 */
-	if (td->zone_size && !td->sequential)
+	if (td->zone_size && !td->sequential && td->nr_files == 1)
 		td->zone_size = 0;
 
 	/*
@@ -151,17 +166,42 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 			td->filetype = FIO_TYPE_CHAR;
 	}
 
+	if (td->odirect)
+		td->io_ops->flags |= FIO_RAWIO;
+
 	if (td->filetype == FIO_TYPE_FILE) {
 		char tmp[PATH_MAX];
+		int len = 0;
+		int i;
 
 		if (td->directory && td->directory[0] != '\0')
-			sprintf(tmp, "%s/%s.%d", td->directory, jobname, td->thread_number);
-		else
-			sprintf(tmp, "%s.%d", jobname, td->thread_number);
-		td->file_name = strdup(tmp);
-	} else
-		td->file_name = strdup(jobname);
+			sprintf(tmp, "%s/", td->directory);
 
+		td->files = malloc(sizeof(struct fio_file) * td->nr_files);
+
+		for_each_file(td, f, i) {
+			memset(f, 0, sizeof(*f));
+			f->fd = -1;
+			f->fileno = i;
+
+			sprintf(tmp + len, "%s.%d.%d", jobname, td->thread_number, i);
+			f->file_name = strdup(tmp);
+		}
+	} else {
+		td->nr_files = 1;
+		td->files = malloc(sizeof(struct fio_file));
+		f = &td->files[0];
+
+		memset(f, 0, sizeof(*f));
+		f->fd = -1;
+		f->file_name = strdup(jobname);
+	}
+
+	for_each_file(td, f, i) {
+		f->file_size = td->total_file_size / td->nr_files;
+		f->file_offset = td->start_offset;
+	}
+		
 	fio_sem_init(&td->mutex, 0);
 
 	td->clat_stat[0].min_val = td->clat_stat[1].min_val = ULONG_MAX;
@@ -183,11 +223,11 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	if (setup_rate(td))
 		goto err;
 
-	if (write_lat_log) {
+	if (td->write_lat_log) {
 		setup_log(&td->slat_log);
 		setup_log(&td->clat_log);
 	}
-	if (write_bw_log)
+	if (td->write_bw_log)
 		setup_log(&td->bw_log);
 
 	if (td->name[0] == '\0')
@@ -196,9 +236,12 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	ddir = td->ddir + (!td->sequential << 1) + (td->iomix << 2);
 
 	if (!terse_output) {
-		if (!job_add_num)
-			fprintf(f_out, "%s: (g=%d): rw=%s, odir=%d, bs=%d-%d, rate=%d, ioengine=%s, iodepth=%d\n", td->name, td->groupid, ddir_str[ddir], td->odirect, td->min_bs, td->max_bs, td->rate, td->io_engine_name, td->iodepth);
-		else if (job_add_num == 1)
+		if (!job_add_num) {
+			if (td->io_ops->flags & FIO_CPUIO)
+				fprintf(f_out, "%s: ioengine=cpu, cpuload=%u, cpucycle=%u\n", td->name, td->cpuload, td->cpucycle);
+			else
+				fprintf(f_out, "%s: (g=%d): rw=%s, odir=%d, bs=%d-%d, rate=%d, ioengine=%s, iodepth=%d\n", td->name, td->groupid, ddir_str[ddir], td->odirect, td->min_bs, td->max_bs, td->rate, td->io_ops->name, td->iodepth);
+		} else if (job_add_num == 1)
 			fprintf(f_out, "...\n");
 	}
 
@@ -233,7 +276,8 @@ err:
 int init_random_state(struct thread_data *td)
 {
 	unsigned long seeds[4];
-	int fd, num_maps, blocks;
+	int fd, num_maps, blocks, i;
+	struct fio_file *f;
 
 	fd = open("/dev/urandom", O_RDONLY);
 	if (fd == -1) {
@@ -259,11 +303,13 @@ int init_random_state(struct thread_data *td)
 	if (td->rand_repeatable)
 		seeds[3] = DEF_RANDSEED;
 
-	blocks = (td->io_size + td->min_bs - 1) / td->min_bs;
-	num_maps = blocks / BLOCKS_PER_MAP;
-	td->file_map = malloc(num_maps * sizeof(long));
-	td->num_maps = num_maps;
-	memset(td->file_map, 0, num_maps * sizeof(long));
+	for_each_file(td, f, i) {
+		blocks = (f->file_size + td->min_bs - 1) / td->min_bs;
+		num_maps = blocks / BLOCKS_PER_MAP;
+		f->file_map = malloc(num_maps * sizeof(long));
+		f->num_maps = num_maps;
+		memset(f->file_map, 0, num_maps * sizeof(long));
+	}
 
 	os_random_seed(seeds[3], &td->random_state);
 	return 0;
@@ -591,34 +637,11 @@ static int str_mem_cb(struct thread_data *td, char *mem)
 
 static int str_ioengine_cb(struct thread_data *td, char *str)
 {
-	if (!strncmp(str, "linuxaio", 8) || !strncmp(str, "aio", 3) ||
-	    !strncmp(str, "libaio", 6)) {
-		strcpy(td->io_engine_name, "libaio");
-		td->io_engine = FIO_LIBAIO;
+	td->io_ops = load_ioengine(td, str);
+	if (td->io_ops)
 		return 0;
-	} else if (!strncmp(str, "posixaio", 8)) {
-		strcpy(td->io_engine_name, "posixaio");
-		td->io_engine = FIO_POSIXAIO;
-		return 0;
-	} else if (!strncmp(str, "sync", 4)) {
-		strcpy(td->io_engine_name, "sync");
-		td->io_engine = FIO_SYNCIO;
-		return 0;
-	} else if (!strncmp(str, "mmap", 4)) {
-		strcpy(td->io_engine_name, "mmap");
-		td->io_engine = FIO_MMAPIO;
-		return 0;
-	} else if (!strncmp(str, "sgio", 4)) {
-		strcpy(td->io_engine_name, "sgio");
-		td->io_engine = FIO_SGIO;
-		return 0;
-	} else if (!strncmp(str, "splice", 6)) {
-		strcpy(td->io_engine_name, "splice");
-		td->io_engine = FIO_SPLICEIO;
-		return 0;
-	}
 
-	log_err("fio: ioengine: { linuxaio, aio, libaio }, posixaio, sync, mmap, sgio, splice\n");
+	log_err("fio: ioengine: { linuxaio, aio, libaio }, posixaio, sync, mmap, sgio, splice, cpu\n");
 	return 1;
 }
 
@@ -726,6 +749,14 @@ int parse_jobs_ini(char *file, int stonewall_flag)
 				fgetpos(f, &off);
 				continue;
 			}
+			if (!check_int(p, "cpuload", &td->cpuload)) {
+				fgetpos(f, &off);
+				continue;
+			}
+			if (!check_int(p, "cpuchunks", &td->cpucycle)) {
+				fgetpos(f, &off);
+				continue;
+			}
 			if (!check_int(p, "thinktime", &td->thinktime)) {
 				fgetpos(f, &off);
 				continue;
@@ -822,6 +853,10 @@ int parse_jobs_ini(char *file, int stonewall_flag)
 				fgetpos(f, &off);
 				continue;
 			}
+			if (!check_int(p, "nrfiles", &td->nr_files)) {
+				fgetpos(f, &off);
+				continue;
+			}
 			if (!check_range_bytes(p, "bsrange", &ul1, &ul2)) {
 				if (ul1 > ul2) {
 					td->max_bs = ul1;
@@ -838,11 +873,11 @@ int parse_jobs_ini(char *file, int stonewall_flag)
 				fgetpos(f, &off);
 				continue;
 			}
-			if (!check_str_bytes(p, "size", &td->file_size)) {
+			if (!check_str_bytes(p, "size", &td->total_file_size)) {
 				fgetpos(f, &off);
 				continue;
 			}
-			if (!check_str_bytes(p, "offset", &td->file_offset)) {
+			if (!check_str_bytes(p, "offset", &td->start_offset)) {
 				fgetpos(f, &off);
 				continue;
 			}
@@ -884,11 +919,6 @@ int parse_jobs_ini(char *file, int stonewall_flag)
 				fgetpos(f, &off);
 				continue;
 			}
-			if (!check_strset(p, "create")) {
-				td->create_file = 1;
-				fgetpos(f, &off);
-				continue;
-			}
 			if (!check_strset(p, "exitall")) {
 				exitall_on_terminate = 1;
 				fgetpos(f, &off);
@@ -904,6 +934,21 @@ int parse_jobs_ini(char *file, int stonewall_flag)
 				fgetpos(f, &off);
 				continue;
 			}
+			if (!check_strset(p, "unlink")) {
+				td->unlink = 1;
+				fgetpos(f, &off);
+				continue;
+			}
+			if (!check_strset(p, "write_bw_log")) {
+				td->write_bw_log = 1;
+				fgetpos(f, &off);
+				continue;
+			}
+			if (!check_strset(p, "write_lat_log")) {
+				td->write_lat_log = 1;
+				fgetpos(f, &off);
+				continue;
+			}
 			if (!check_strstore(p, "iolog", tmpbuf)) {
 				if (td->write_iolog) {
 					log_err("fio: read iolog overrides given write_iolog\n");
@@ -987,13 +1032,10 @@ static int fill_def_thread(void)
 	def_thread.bs = DEF_BS;
 	def_thread.min_bs = -1;
 	def_thread.max_bs = -1;
-	def_thread.io_engine = DEF_IO_ENGINE;
-	strcpy(def_thread.io_engine_name, DEF_IO_ENGINE_NAME);
 	def_thread.odirect = DEF_ODIRECT;
 	def_thread.ratecycle = DEF_RATE_CYCLE;
 	def_thread.sequential = DEF_SEQUENTIAL;
 	def_thread.timeout = def_timeout;
-	def_thread.create_file = DEF_CREATE;
 	def_thread.overwrite = DEF_OVERWRITE;
 	def_thread.invalidate_cache = DEF_INVALIDATE;
 	def_thread.sync_io = DEF_SYNCIO;
@@ -1010,6 +1052,10 @@ static int fill_def_thread(void)
 	def_thread.rwmixread = DEF_RWMIX_READ;
 	def_thread.nice = DEF_NICE;
 	def_thread.rand_repeatable = DEF_RAND_REPEAT;
+	def_thread.nr_files = DEF_NR_FILES;
+	def_thread.unlink = DEF_UNLINK;
+	def_thread.write_bw_log = write_bw_log;
+	def_thread.write_lat_log = write_lat_log;
 #ifdef FIO_HAVE_DISK_UTIL
 	def_thread.do_disk_util = 1;
 #endif
@@ -1017,7 +1063,7 @@ static int fill_def_thread(void)
 	return 0;
 }
 
-static void usage(char *name)
+static void usage(void)
 {
 	printf("%s\n", fio_version_string);
 	printf("\t-o Write output to file\n");
@@ -1060,7 +1106,7 @@ static int parse_cmd_line(int argc, char *argv[])
 				idx = optind;
 				break;
 			case 'h':
-				usage(argv[0]);
+				usage();
 				exit(0);
 			case 'v':
 				printf("%s\n", fio_version_string);
@@ -1144,7 +1190,7 @@ int parse_options(int argc, char *argv[])
 	job_files = parse_cmd_line(argc, argv);
 	if (!job_files) {
 		log_err("Need job file(s)\n");
-		usage(argv[0]);
+		usage();
 		return 1;
 	}
 
