@@ -47,12 +47,4 @@
 #define OS_O_DIRECT			O_DIRECT
 #endif
 
-struct thread_data;
-extern int fio_libaio_init(struct thread_data *);
-extern int fio_posixaio_init(struct thread_data *);
-extern int fio_syncio_init(struct thread_data *);
-extern int fio_mmapio_init(struct thread_data *);
-extern int fio_sgio_init(struct thread_data *);
-extern int fio_spliceio_init(struct thread_data *);
-
 #endif
