@@ -187,6 +187,7 @@ static int find_block_dir(dev_t dev, char *path)
 
 void init_disk_util(struct thread_data *td)
 {
+	struct fio_file *f;
 	struct stat st;
 	char foo[256], tmp[256];
 	dev_t dev;
@@ -195,7 +196,11 @@ void init_disk_util(struct thread_data *td)
 	if (!td->do_disk_util)
 		return;
 
-	if (!stat(td->file_name, &st)) {
+	/*
+	 * Just use the same file, they are on the same device.
+	 */
+	f = &td->files[0];
+	if (!stat(f->file_name, &st)) {
 		if (S_ISBLK(st.st_mode))
 			dev = st.st_rdev;
 		else
@@ -204,7 +209,7 @@ void init_disk_util(struct thread_data *td)
 		/*
 		 * must be a file, open "." in that path
 		 */
-		strcpy(foo, td->file_name);
+		strcpy(foo, f->file_name);
 		p = dirname(foo);
 		if (stat(p, &st)) {
 			perror("disk util stat");
@@ -234,10 +239,13 @@ void init_disk_util(struct thread_data *td)
 			log_err("unknown sysfs layout\n");
 			return;
 		}
-		sprintf(foo, "%s", p);
+		strcpy(tmp, p);
+		sprintf(foo, "%s", tmp);
 	}
 
-	td->sysfs_root = strdup(foo);
+	if (td->ioscheduler)
+		td->sysfs_root = strdup(foo);
+
 	disk_util_add(dev, foo);
 }
 
@@ -294,7 +302,7 @@ static void show_group_stats(struct group_run_stats *rs, int id)
 static void show_disk_util(void)
 {
 	struct disk_util_stat *dus;
-	struct list_head *entry;
+	struct list_head *entry, *next;
 	struct disk_util *du;
 	double util;
 
@@ -310,6 +318,16 @@ static void show_disk_util(void)
 
 		fprintf(f_out, "  %s: ios=%u/%u, merge=%u/%u, ticks=%u/%u, in_queue=%u, util=%3.2f%%\n", du->name, dus->ios[0], dus->ios[1], dus->merges[0], dus->merges[1], dus->ticks[0], dus->ticks[1], dus->time_in_queue, util);
 	}
+
+	/*
+	 * now free the list
+	 */
+	list_for_each_safe(entry, next, &disk_list) {
+		list_del(entry);
+		du = list_entry(entry, struct disk_util, list);
+		free(du->name);
+		free(du);
+	}
 }
 
 static void show_ddir_status(struct thread_data *td, struct group_run_stats *rs,
@@ -440,11 +458,9 @@ void show_run_stats(void)
 		rs->min_bw[1] = rs->min_run[1] = ~0UL;
 	}
 
-	for (i = 0; i < thread_number; i++) {
+	for_each_td(td, i) {
 		unsigned long long rbw, wbw;
 
-		td = &threads[i];
-
 		if (td->error) {
 			fprintf(f_out, "%s: %s\n", td->name, td->verror);
 			continue;
@@ -495,8 +511,7 @@ void show_run_stats(void)
 	if (!terse_output)
 		printf("\n");
 
-	for (i = 0; i < thread_number; i++) {
-		td = &threads[i];
+	for_each_td(td, i) {
 		rs = &runstats[td->groupid];
 
 		if (terse_output)
@@ -511,6 +526,8 @@ void show_run_stats(void)
 
 		show_disk_util();
 	}
+
+	free(runstats);
 }
 
 static inline void add_stat_sample(struct io_stat *is, unsigned long val)
