@@ -3,6 +3,8 @@
 
 #include "fio.h"
 
+static struct timeval genesis;
+
 unsigned long utime_since(struct timeval *s, struct timeval *e)
 {
 	double sec, usec;
@@ -60,7 +62,7 @@ unsigned long time_since_now(struct timeval *s)
 /*
  * busy looping version for the last few usec
  */
-static void __usec_sleep(unsigned int usec)
+void __usec_sleep(unsigned int usec)
 {
 	struct timeval start;
 
@@ -120,3 +122,13 @@ void rate_throttle(struct thread_data *td, unsigned long time_spent,
 		td->rate_pending_usleep -= overtime;
 	}
 }
+
+unsigned long mtime_since_genesis(void)
+{
+	return mtime_since_now(&genesis);
+}
+
+void time_init(void)
+{
+	gettimeofday(&genesis, NULL);
+}
